﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameManager : MonoBehaviour
{
    // GameOverUI
    public GameObject gameOverUI;

    public TimeManager timeManager;

    public SceneController sceneController;
    public CameraController cameraController;

    public GameObject wifeObjs;
    public GameObject wifeFront;
    public GameObject wifeFrontEyeLaser;

    public GameObject wifeExcMark1;
    public GameObject wifeExcMark2;
    public GameObject wifeExcMark3;
    public GameObject wifeExcMark4;
    public GameObject wifeExcMark5;

    public GameObject wifeCamera;

    public GameObject manExcMark;

    public int precious = 0;
    public int currentSceneNumber = 1;

    //public GameObject preciousPref;

    [Header("Difficulty")]
    public int startHealth = 3;
    private int currnetHealth;
    public HealthUI healthUI;
    public int treasureStackCnt = 1;
    private int treasureNextIdx = 0;
    public int zoomOutCount = 10;
    public int endSceneCount = 100;

    public float warningWaitFor = 3f;
    public float Action_3_SheLooksAtMe_waitFor = 1f;

    public ArmAnimator armAnimator;

    public GameObject treasureArr;
    SpriteRenderer[] treasures;


    bool reserved = false;
    bool wifeIsHere = false;

    bool end = false;

    bool isPressed = false;

    bool isPaused = false;

    [Header("UI")]
    public GameObject ButtonUp;
    public GameObject ButtonDown;
    public GaugeMovement gaugeMovement;
    public Text ScoreText;

    public GameObject preciousAnimePref;
    public GameObject preciousParent;

    [Header("Sound")]
    public AudioClip damagedAudio;
    public AudioClip lookingAudio;
    public AudioSource sfx;
    

    
    [Header("Development")]

    public bool debugOn = false;

    private void Awake()
    {
        treasures = treasureArr.GetComponentsInChildren<SpriteRenderer>();
    }

    void Start()
    {
        currnetHealth = startHealth;
        healthUI.setCurrentHealth(currnetHealth);
        if (debugOn)
        {
            treasureStackCnt = 1;
        }
        else
        {
            // Three coroutine looping
            // 1. Action_1_Peaceful
            // 2. Action_2_Warning
            // 3. Action_3_SheLooksAtMe
            Action_1_Peaceful();
        }
    }

    // Update is called once per frame
    void Update()
    {
        if (isPressed && !end)
        {
            if (wifeIsHere)
            {
                Damaged();

                return;
            }
        }
        if (precious > endSceneCount)
        {
            end = true;
            sceneController.Select("PlayScene" + (currentSceneNumber + 1));
        }


    }

    public void Pressed()
    {
        isPressed = true;
        armAnimator.MoveArm();
        ButtonUp.SetActive(false);
        ButtonDown.SetActive(true);
        gaugeMovement.MoveGauge();
        //gaugeMovement.ResetPosition();

    }

    private void Shoot()
    {

        if (debugOn || gaugeMovement.IsHit())
        {
            armAnimator.RotateArm();
            ButtonUp.SetActive(true);
            ButtonDown.SetActive(false);
            isPressed = false;
            gaugeMovement.PauseGauge();
            GameObject generatedPreciousAnime = Instantiate(preciousAnimePref, preciousParent.transform);


            //GameObject generatedPrecious = Instantiate(preciousPref);
            //generatedPrecious.transform.localScale = new Vector3(Random.Range(0.1f, 0.3f), Random.Range(0.1f, 0.3f), Random.Range(0.1f, 0.3f));
            precious++;
            ScoreText.text = precious.ToString();

            if (precious / treasureStackCnt >= treasureNextIdx + 1)
            {
                if (treasures.Length > treasureNextIdx)
                {
                    treasures[treasureNextIdx].enabled = true;
                }
                treasureNextIdx++;
                if (treasureNextIdx > 37)
                {
                    cameraController.setCameraPositionY(10f);
                    wifeCamera.SetActive(true);
                } else if (treasureNextIdx > 30)
                {
                    cameraController.setCameraPositionY(5f);
                    wifeCamera.SetActive(true);
                } else if (treasureNextIdx > 23)
                {
                    cameraController.setCameraPositionY(3f);
                }
            }
            StartCoroutine(RemoveObj(generatedPreciousAnime));
        }
        else
        {
            UnPressed();
        }
        ButtonUp.SetActive(true);
        ButtonDown.SetActive(false);
    }

    IEnumerator RemoveObj(GameObject obj)
    {
        float t = 0f;
        while (t < 10f)
        {
            t += Time.deltaTime;
            //if (t > 1f)
            //{
            //    obj.transform.position.Set(
            //        obj.transform.position.x + Random.Range(1f, 3f),
            //        obj.transform.position.y + Random.Range(1f, 3f),
            //        obj.transform.position.z
            //    );
            //}
            yield return 0;
        }
        Destroy(obj);
    }
    public void UnPressed()
    {
        ButtonUp.SetActive(true);
        ButtonDown.SetActive(false);
        isPressed = false;
        gaugeMovement.PauseGauge();
        armAnimator.DoNothing();
    }
    private void Damaged()
    {
        wifeFront.SetActive(false);
        wifeFrontEyeLaser.transform.position = wifeFront.transform.position;
        wifeFrontEyeLaser.SetActive(true);

        //StartCoroutine(PauseGame(3f));
        
        UnPressed();

        StartCoroutine(ManDetectedAction(1f));
        if (currnetHealth <= 0)
        {
            GameOver();
            return;
        }
        currnetHealth--;

        healthUI.setCurrentHealth(currnetHealth);
        sfx.clip = damagedAudio;
        sfx.Play();
    }

    public IEnumerator ManDetectedAction(float wait)
    {
        manExcMark.SetActive(true);
        float t = 0f;

        while (t < wait)
        {
            t += Time.deltaTime;
            yield return 0;
        }
        manExcMark.SetActive(false);
        armAnimator.DoNothing();

        wifeIsHere = false;
        reserved = false;
        Action_1_Peaceful();
    }

    public IEnumerator PauseGame(float wait)
    {
        //TO-DO 
        //Time.timeScale = 0.4f;
        //
        timeManager.DoSlowMo();
        

        Debug.Log("Inside PauseGame()");
        GameObject[] childParticles = preciousParent.GetComponentsInChildren<GameObject>();

        foreach (GameObject childParticle in childParticles)
        {
            childParticle.GetComponent<Animator>().enabled = false;
        }

        yield return new WaitForSeconds(wait);
    }

    private void GameOver()
    {
        Debug.Log("GameOver");
        gameOverUI.SetActive(true);
    }

    private void Action_2_Warning()
    {
        wifeObjs.GetComponent<Animator>().enabled = false;
        Debug.Log("WifeHi");
        if (!reserved)
        {
            reserved = true;
            wifeExcMark1.SetActive(true);
            wifeExcMark2.SetActive(true);
            wifeExcMark3.SetActive(true);
            wifeExcMark4.SetActive(true);
            wifeExcMark5.SetActive(true);

            StartCoroutine(Action_2_WarningCoroutineFn(warningWaitFor));
        }
    }

    private void Action_3_SheLooksAtMe()
    {
        wifeObjs.GetComponent<Animator>().enabled = false;
        StartCoroutine(Action_3_SheLooksAtMeCoroutineFn(Action_3_SheLooksAtMe_waitFor));
    }



    private void Action_1_Peaceful()
    {
        Debug.Log("Peaceful");
        float peacefulWaitFor = Random.Range(2f, 15f);
        wifeFront.SetActive(false);
        wifeFrontEyeLaser.SetActive(false);
        wifeObjs.GetComponent<Animator>().enabled = true;
        wifeExcMark1.SetActive(false);
        wifeExcMark2.SetActive(false);
        wifeExcMark3.SetActive(false);
        wifeExcMark4.SetActive(false);
        wifeExcMark5.SetActive(false);
        StartCoroutine(Action_1_PeacefulCoroutineFn(peacefulWaitFor));
    }

    IEnumerator Action_2_WarningCoroutineFn(float wait)
    {
        
        float t = 0f;


        while (t < wait)
        {
            t += Time.deltaTime;
            yield return 0;
        }

        Transform currentActiveTransform = getActiveWifeName();
        wifeFront.SetActive(true);
        wifeFront.transform.position = currentActiveTransform.position;
        currentActiveTransform.gameObject.SetActive(false);
        wifeObjs.GetComponent<Animator>().enabled = false;
        wifeIsHere = true;

        sfx.clip = lookingAudio;
        sfx.Play();
        
        Action_3_SheLooksAtMe();
    }

    IEnumerator Action_3_SheLooksAtMeCoroutineFn(float wait)
    {
        float t = 0f;

        while (t < wait)
        {
            t += Time.deltaTime;
            yield return 0;
        }



        wifeIsHere = false;
        reserved = false;
        Action_1_Peaceful();
    }

    IEnumerator Action_1_PeacefulCoroutineFn(float wait)
    {

        float t = 0f;

        while (t < wait)
        {
            t += Time.deltaTime;
            yield return 0;
        }

        Action_2_Warning();
    }

    private Transform getActiveWifeName()
    {

        foreach (Transform child in wifeObjs.transform)
        {
            if (child.gameObject.activeSelf)
                return child;
        }
        return null;
    }

    public void OnRedButtonPress()
    {
        Pressed();

    }
    public void OnRedButtonRelease()
    {
        if (!isPressed) return;
        UnPressed();
        if (!end) Shoot();
    }
}
